@@ -22,10 +22,7 @@ import time
 
 from struct import pack, unpack
 
-from kmip.core.errors import ErrorStrings
-
 from kmip.core import enums
-from kmip.core import errors
 from kmip.core import exceptions
 from kmip.core import utils
 
@@ -45,7 +42,7 @@ class Base(object):
     def is_oversized(self, stream):
         extra = len(stream.peek())
         if extra > 0:
-            raise errors.StreamNotEmptyError(Base.__name__, extra)
+            raise exceptions.StreamNotEmptyError(Base.__name__, extra)
 
     def read_tag(self, istream):
         # Read in the bytes for the tag
@@ -56,8 +53,12 @@ class Base(object):
 
         # Verify that the tag matches for the current object
         if enum_tag is not self.tag:
-            raise errors.ReadValueError(Base.__name__, 'tag',
-                                        hex(self.tag.value), hex(tag))
+            raise exceptions.ReadValueError(
+                Base.__name__,
+                'tag',
+                hex(self.tag.value),
+                hex(tag)
+            )
 
     def read_type(self, istream):
         # Read in the bytes for the type
@@ -65,15 +66,23 @@ class Base(object):
         num_bytes = len(tts)
         if num_bytes != self.TYPE_SIZE:
             min_bytes = 'a minimum of {0} bytes'.format(self.TYPE_SIZE)
-            raise errors.ReadValueError(Base.__name__, 'type', min_bytes,
-                                        '{0} bytes'.format(num_bytes))
+            raise exceptions.ReadValueError(
+                Base.__name__,
+                'type',
+                min_bytes,
+                '{0} bytes'.format(num_bytes)
+            )
         typ = unpack('!B', tts)[0]
 
         enum_typ = enums.Types(typ)
 
         if enum_typ is not self.type:
-            raise errors.ReadValueError(Base.__name__, 'type',
-                                        self.type.value, typ)
+            raise exceptions.ReadValueError(
+                Base.__name__,
+                'type',
+                self.type.value,
+                typ
+            )
 
     def read_length(self, istream):
         # Read in the bytes for the length
@@ -81,14 +90,18 @@ class Base(object):
         num_bytes = len(lst)
         if num_bytes != self.LENGTH_SIZE:
             min_bytes = 'a minimum of {0} bytes'.format(self.LENGTH_SIZE)
-            raise errors.ReadValueError(Base.__name__, 'length', min_bytes,
-                                        '{0} bytes'.format(num_bytes))
+            raise exceptions.ReadValueError(
+                Base.__name__,
+                'length',
+                min_bytes,
+                '{0} bytes'.format(num_bytes)
+            )
         self.length = unpack('!I', lst)[0]
 
     def read_value(self, istream):
         raise NotImplementedError()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         self.read_tag(istream)
         self.read_type(istream)
         self.read_length(istream)
@@ -99,26 +112,30 @@ class Base(object):
 
     def write_type(self, ostream):
         if type(self.type) is not enums.Types:
-            msg = ErrorStrings.BAD_EXP_RECV
+            msg = exceptions.ErrorStrings.BAD_EXP_RECV
             raise TypeError(msg.format(Base.__name__, 'type',
                                        enums.Types, type(self.type)))
         ostream.write(pack('!B', self.type.value))
 
     def write_length(self, ostream):
         if type(self.length) is not int:
-            msg = ErrorStrings.BAD_EXP_RECV
+            msg = exceptions.ErrorStrings.BAD_EXP_RECV
             raise TypeError(msg.format(Base.__name__, 'length',
                                        int, type(self.length)))
         num_bytes = utils.count_bytes(self.length)
         if num_bytes > self.LENGTH_SIZE:
-            raise errors.WriteOverflowError(Base.__name__, 'length',
-                                            self.LENGTH_SIZE, num_bytes)
+            raise exceptions.WriteOverflowError(
+                Base.__name__,
+                'length',
+                self.LENGTH_SIZE,
+                num_bytes
+            )
         ostream.write(pack('!I', self.length))
 
     def write_value(self, ostream):
         raise NotImplementedError()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         self.write_tag(ostream)
         self.write_type(ostream)
         self.write_length(ostream)
@@ -188,28 +205,36 @@ class Integer(Base):
 
     def read_value(self, istream):
         if self.length is not self.LENGTH:
-            raise errors.ReadValueError(Integer.__name__, 'length',
-                                        self.LENGTH, self.length)
+            raise exceptions.ReadValueError(
+                Integer.__name__,
+                'length',
+                self.LENGTH,
+                self.length
+            )
 
         self.value = unpack(self.pack_string, istream.read(self.length))[0]
         pad = unpack(self.pack_string, istream.read(self.padding_length))[0]
 
-        if pad is not 0:
-            raise errors.ReadValueError(Integer.__name__, 'pad', 0,
-                                        pad)
+        if pad != 0:
+            raise exceptions.ReadValueError(
+                Integer.__name__,
+                'pad',
+                0,
+                pad
+            )
         self.validate()
 
-    def read(self, istream):
-        super(Integer, self).read(istream)
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(Integer, self).read(istream, kmip_version=kmip_version)
         self.read_value(istream)
 
-    def write_value(self, ostream):
+    def write_value(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         ostream.write(pack(self.pack_string, self.value))
         ostream.write(pack(self.pack_string, 0))
 
-    def write(self, ostream):
-        super(Integer, self).write(ostream)
-        self.write_value(ostream)
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(Integer, self).write(ostream, kmip_version=kmip_version)
+        self.write_value(ostream, kmip_version=kmip_version)
 
     def validate(self):
         """
@@ -303,19 +328,22 @@ class LongInteger(Base):
 
         self.validate()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the encoding of the LongInteger from the input stream.
 
         Args:
             istream (stream): A buffer containing the encoded bytes of a
                 LongInteger. Usually a BytearrayStream object. Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
 
         Raises:
             InvalidPrimitiveLength: if the long integer encoding read in has
                 an invalid encoded length.
         """
-        super(LongInteger, self).read(istream)
+        super(LongInteger, self).read(istream, kmip_version=kmip_version)
 
         if self.length is not LongInteger.LENGTH:
             raise exceptions.InvalidPrimitiveLength(
@@ -326,15 +354,18 @@ class LongInteger(Base):
         self.value = unpack('!q', istream.read(self.length))[0]
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the encoding of the LongInteger to the output stream.
 
         Args:
             ostream (stream): A buffer to contain the encoded bytes of a
                 LongInteger. Usually a BytearrayStream object. Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(LongInteger, self).write(ostream)
+        super(LongInteger, self).write(ostream, kmip_version=kmip_version)
         ostream.write(pack('!q', self.value))
 
     def validate(self):
@@ -395,7 +426,7 @@ class BigInteger(Base):
 
         self.validate()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the encoding of the BigInteger from the input stream.
 
@@ -403,12 +434,15 @@ class BigInteger(Base):
             istream (stream): A buffer containing the encoded bytes of the
                 value of a BigInteger. Usually a BytearrayStream object.
                 Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
 
         Raises:
             InvalidPrimitiveLength: if the big integer encoding read in has
                 an invalid encoded length.
         """
-        super(BigInteger, self).read(istream)
+        super(BigInteger, self).read(istream, kmip_version=kmip_version)
 
         # Check for a valid length before even trying to parse the value.
         if self.length % 8:
@@ -442,7 +476,7 @@ class BigInteger(Base):
         # Convert the value back to an integer and reapply the sign.
         self.value = int(binary, 2) * sign
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the encoding of the BigInteger to the output stream.
 
@@ -450,6 +484,9 @@ class BigInteger(Base):
             ostream (Stream): A buffer to contain the encoded bytes of a
                 BigInteger object. Usually a BytearrayStream object.
                 Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
         # Convert the value to binary and pad it as needed.
         binary = "{0:b}".format(abs(self.value))
@@ -472,7 +509,7 @@ class BigInteger(Base):
             hexadecimal += struct.pack('!B', byte)
 
         self.length = len(hexadecimal)
-        super(BigInteger, self).write(ostream)
+        super(BigInteger, self).write(ostream, kmip_version=kmip_version)
         ostream.write(hexadecimal)
 
     def validate(self):
@@ -543,13 +580,16 @@ class Enumeration(Base):
 
         self.validate()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the encoding of the Enumeration from the input stream.
 
         Args:
             istream (stream): A buffer containing the encoded bytes of an
                 Enumeration. Usually a BytearrayStream object. Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
 
         Raises:
             InvalidPrimitiveLength: if the Enumeration encoding read in has an
@@ -557,7 +597,7 @@ class Enumeration(Base):
             InvalidPaddingBytes: if the Enumeration encoding read in does not
                 use zeroes for its padding bytes.
         """
-        super(Enumeration, self).read(istream)
+        super(Enumeration, self).read(istream, kmip_version=kmip_version)
 
         # Check for a valid length before even trying to parse the value.
         if self.length != Enumeration.LENGTH:
@@ -570,20 +610,23 @@ class Enumeration(Base):
         pad = unpack('!I', istream.read(Enumeration.LENGTH))[0]
 
         # Verify that the padding bytes are zero bytes.
-        if pad is not 0:
+        if pad != 0:
             raise exceptions.InvalidPaddingBytes("padding bytes must be zero")
 
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the encoding of the Enumeration to the output stream.
 
         Args:
             ostream (stream): A buffer to contain the encoded bytes of an
                 Enumeration. Usually a BytearrayStream object. Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(Enumeration, self).write(ostream)
+        super(Enumeration, self).write(ostream, kmip_version=kmip_version)
         ostream.write(pack('!I', self.value.value))
         ostream.write(pack('!I', 0))
 
@@ -664,7 +707,7 @@ class Boolean(Base):
 
         self.validate()
 
-    def read_value(self, istream):
+    def read_value(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the value of the Boolean object from the input stream.
 
@@ -672,6 +715,9 @@ class Boolean(Base):
             istream (Stream): A buffer containing the encoded bytes of the
                 value of a Boolean object. Usually a BytearrayStream object.
                 Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
 
         Raises:
             ValueError: if the read boolean value is not a 0 or 1.
@@ -691,18 +737,21 @@ class Boolean(Base):
 
         self.validate()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the encoding of the Boolean object from the input stream.
 
         Args:
             istream (Stream): A buffer containing the encoded bytes of a
                 Boolean object. Usually a BytearrayStream object. Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(Boolean, self).read(istream)
-        self.read_value(istream)
+        super(Boolean, self).read(istream, kmip_version=kmip_version)
+        self.read_value(istream, kmip_version=kmip_version)
 
-    def write_value(self, ostream):
+    def write_value(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the value of the Boolean object to the output stream.
 
@@ -710,6 +759,9 @@ class Boolean(Base):
             ostream (Stream): A buffer to contain the encoded bytes of the
                 value of a Boolean object. Usually a BytearrayStream object.
                 Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
         try:
             ostream.write(pack('!Q', self.value))
@@ -717,16 +769,19 @@ class Boolean(Base):
             self.logger.error("Error writing boolean value to buffer")
             raise
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the encoding of the Boolean object to the output stream.
 
         Args:
             ostream (Stream): A buffer to contain the encoded bytes of a
                 Boolean object. Usually a BytearrayStream object. Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(Boolean, self).write(ostream)
-        self.write_value(ostream)
+        super(Boolean, self).write(ostream, kmip_version=kmip_version)
+        self.write_value(ostream, kmip_version=kmip_version)
 
     def validate(self):
         """
@@ -783,7 +838,7 @@ class TextString(Base):
             self.length = None
             self.padding_length = None
 
-    def read_value(self, istream):
+    def read_value(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         # Read string text
         self.value = ''
         for _ in range(self.length):
@@ -798,16 +853,20 @@ class TextString(Base):
         if self.padding_length < self.PADDING_SIZE:
             for _ in range(self.padding_length):
                 pad = unpack('!B', istream.read(1))[0]
-                if pad is not 0:
-                    raise errors.ReadValueError(TextString.__name__, 'pad', 0,
-                                                pad)
+                if pad != 0:
+                    raise exceptions.ReadValueError(
+                        TextString.__name__,
+                        'pad',
+                        0,
+                        pad
+                    )
 
-    def read(self, istream):
-        super(TextString, self).read(istream)
-        self.read_value(istream)
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(TextString, self).read(istream, kmip_version=kmip_version)
+        self.read_value(istream, kmip_version=kmip_version)
         self.validate()
 
-    def write_value(self, ostream):
+    def write_value(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         # Write string to stream
         for char in self.value:
             ostream.write(pack(self.BYTE_FORMAT, char.encode()))
@@ -816,9 +875,9 @@ class TextString(Base):
         for _ in range(self.padding_length):
             ostream.write(pack('!B', 0))
 
-    def write(self, ostream):
-        super(TextString, self).write(ostream)
-        self.write_value(ostream)
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(TextString, self).write(ostream, kmip_version=kmip_version)
+        self.write_value(ostream, kmip_version=kmip_version)
 
     def validate(self):
         self.__validate()
@@ -826,7 +885,7 @@ class TextString(Base):
     def __validate(self):
         if self.value is not None:
             if not isinstance(self.value, six.string_types):
-                msg = ErrorStrings.BAD_EXP_RECV
+                msg = exceptions.ErrorStrings.BAD_EXP_RECV
                 raise TypeError(msg.format('TextString', 'value', str,
                                            type(self.value)))
 
@@ -873,7 +932,7 @@ class ByteString(Base):
             self.length = None
             self.padding_length = None
 
-    def read_value(self, istream):
+    def read_value(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         # Read bytes into bytearray
         data = bytearray()
         for _ in range(self.length):
@@ -889,15 +948,19 @@ class ByteString(Base):
         if self.padding_length < self.PADDING_SIZE:
             for _ in range(self.padding_length):
                 pad = unpack('!B', istream.read(1))[0]
-                if pad is not 0:
-                    raise errors.ReadValueError(TextString.__name__, 'pad', 0,
-                                                pad)
+                if pad != 0:
+                    raise exceptions.ReadValueError(
+                        TextString.__name__,
+                        'pad',
+                        0,
+                        pad
+                    )
 
-    def read(self, istream):
-        super(ByteString, self).read(istream)
-        self.read_value(istream)
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(ByteString, self).read(istream, kmip_version=kmip_version)
+        self.read_value(istream, kmip_version=kmip_version)
 
-    def write_value(self, ostream):
+    def write_value(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         # Write bytes to stream
         data = bytearray(self.value)
         for byte in data:
@@ -907,9 +970,9 @@ class ByteString(Base):
         for _ in range(self.padding_length):
             ostream.write(pack('!B', 0))
 
-    def write(self, ostream):
-        super(ByteString, self).write(ostream)
-        self.write_value(ostream)
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
+        super(ByteString, self).write(ostream, kmip_version=kmip_version)
+        self.write_value(ostream, kmip_version=kmip_version)
 
     def validate(self):
         self.__validate()
@@ -919,7 +982,7 @@ class ByteString(Base):
         if self.value is not None:
             data_type = type(self.value)
             if data_type is not bytes:
-                msg = ErrorStrings.BAD_EXP_RECV
+                msg = exceptions.ErrorStrings.BAD_EXP_RECV
                 raise TypeError(msg.format('ByteString', 'value', bytes,
                                            data_type))
 
@@ -998,7 +1061,7 @@ class Interval(Base):
 
         self.validate()
 
-    def read(self, istream):
+    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Read the encoding of the Interval from the input stream.
 
@@ -1006,6 +1069,9 @@ class Interval(Base):
             istream (stream): A buffer containing the encoded bytes of the
                 value of an Interval. Usually a BytearrayStream object.
                 Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be decoded. Optional,
+                defaults to KMIP 1.0.
 
         Raises:
             InvalidPrimitiveLength: if the Interval encoding read in has an
@@ -1013,7 +1079,7 @@ class Interval(Base):
             InvalidPaddingBytes: if the Interval encoding read in does not use
                 zeroes for its padding bytes.
         """
-        super(Interval, self).read(istream)
+        super(Interval, self).read(istream, kmip_version=kmip_version)
 
         # Check for a valid length before even trying to parse the value.
         if self.length != Interval.LENGTH:
@@ -1025,20 +1091,23 @@ class Interval(Base):
         pad = unpack('!I', istream.read(Interval.LENGTH))[0]
 
         # Verify that the padding bytes are zero bytes.
-        if pad is not 0:
+        if pad != 0:
             raise exceptions.InvalidPaddingBytes("padding bytes must be zero")
 
         self.validate()
 
-    def write(self, ostream):
+    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
         """
         Write the encoding of the Interval to the output stream.
 
         Args:
             ostream (stream): A buffer to contain the encoded bytes of an
                 Interval. Usually a BytearrayStream object. Required.
+            kmip_version (KMIPVersion): An enumeration defining the KMIP
+                version with which the object will be encoded. Optional,
+                defaults to KMIP 1.0.
         """
-        super(Interval, self).write(ostream)
+        super(Interval, self).write(ostream, kmip_version=kmip_version)
         ostream.write(pack('!I', self.value))
         ostream.write(pack('!I', 0))
 
