@@ -44,9 +44,16 @@ if __name__ == '__main__':
     algorithm = getattr(enums.CryptographicAlgorithm, algorithm, None)
 
     # Build the client and connect to the server
-    with client.ProxyKmipClient(config=config) as client:
+    with client.ProxyKmipClient(
+            config=config,
+            config_file=opts.config_file
+    ) as client:
         try:
-            uid = client.create(algorithm, length)
+            uid = client.create(
+                algorithm,
+                length,
+                operation_policy_name=opts.operation_policy_name
+            )
             logger.info("Successfully created symmetric key with ID: "
                         "{0}".format(uid))
         except Exception as e:
