@@ -43,9 +43,18 @@ if __name__ == '__main__':
     algorithm = getattr(enums.CryptographicAlgorithm, algorithm, None)
 
     # Build the client and connect to the server
-    with client.ProxyKmipClient(config=config) as client:
+    with client.ProxyKmipClient(
+            config=config,
+            config_file=opts.config_file
+    ) as client:
         try:
-            public_uid, private_uid = client.create_key_pair(algorithm, length)
+            public_uid, private_uid = client.create_key_pair(
+                algorithm,
+                length,
+                operation_policy_name=opts.operation_policy_name,
+                public_usage_mask=[enums.CryptographicUsageMask.VERIFY],
+                private_usage_mask=[enums.CryptographicUsageMask.SIGN]
+            )
             logger.info("Successfully created public key with ID: {0}".format(
                 public_uid))
             logger.info("Successfully created private key with ID: {0}".format(
