@@ -37,7 +37,10 @@ if __name__ == '__main__':
         sys.exit()
 
     # Build the client and connect to the server
-    with client.ProxyKmipClient(config=config) as client:
+    with client.ProxyKmipClient(
+            config=config,
+            config_file=opts.config_file
+    ) as client:
         try:
             client.destroy(uid)
             logger.info("Successfully destroyed secret with ID: {0}".format(
