@@ -36,9 +36,13 @@ if __name__ == '__main__':
     name = 'Demo Opaque Object'
 
     obj = objects.OpaqueObject(value, opaque_type, name)
+    obj.operation_policy_name = opts.operation_policy_name
 
     # Build the client and connect to the server
-    with client.ProxyKmipClient(config=config) as client:
+    with client.ProxyKmipClient(
+            config=config,
+            config_file=opts.config_file
+    ) as client:
         try:
             uid = client.register(obj)
             logger.info("Successfully registered opaque object with ID: "
