@@ -57,9 +57,13 @@ if __name__ == '__main__':
 
     key = objects.PublicKey(
         algorithm, length, value, format_type, usage_mask, name)
+    key.operation_policy_name = opts.operation_policy_name
 
     # Build the client and connect to the server
-    with client.ProxyKmipClient(config=config) as client:
+    with client.ProxyKmipClient(
+            config=config,
+            config_file=opts.config_file
+    ) as client:
         try:
             uid = client.register(key)
             logger.info("Successfully registered public key with ID: "
