@@ -13,6 +13,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from kmip.core import enums
 from kmip.core.enums import AttributeType
 from kmip.core.enums import CredentialType
 from kmip.core.enums import CryptographicAlgorithm
@@ -73,7 +74,7 @@ if __name__ == '__main__':
         credential = credential_factory.create_credential(credential_type,
                                                           credential_value)
     # Build the client and connect to the server
-    client = KMIPProxy(config=config)
+    client = KMIPProxy(config=config, config_file=opts.config_file)
     client.open()
 
     # Build the different object attributes
@@ -108,6 +109,14 @@ if __name__ == '__main__':
     name = Attribute(attribute_name=name, attribute_value=value)
 
     attributes = [algorithm_obj, usage_mask, length_obj, name]
+
+    if opts.operation_policy_name is not None:
+        opn = attribute_factory.create_attribute(
+            enums.AttributeType.OPERATION_POLICY_NAME,
+            opts.operation_policy_name
+        )
+        attributes.append(opn)
+
     template_attribute = TemplateAttribute(attributes=attributes)
 
     # Create the SYMMETRIC_KEY object
@@ -121,8 +130,8 @@ if __name__ == '__main__':
 
     if result.result_status.value == ResultStatus.SUCCESS:
         logger.info('created object type: {0}'.format(
-            result.object_type.value))
-        logger.info('created UUID: {0}'.format(result.uuid.value))
+            result.object_type))
+        logger.info('created UUID: {0}'.format(result.uuid))
         logger.info('created template attribute: {0}'.
                     format(result.template_attribute))
     else:
