@@ -41,12 +41,11 @@ if __name__ == '__main__':
     protocol_versions = list()
     if opts.protocol_versions is not None:
         for version in re.split(',| ', opts.protocol_versions):
-            mm = re.split('\.', version)
-            protocol_versions.append(ProtocolVersion.create(int(mm[0]),
-                                                            int(mm[1])))
+            mm = re.split(r'\.', version)
+            protocol_versions.append(ProtocolVersion(int(mm[0]), int(mm[1])))
 
     # Build the client and connect to the server
-    client = KMIPProxy(config=config)
+    client = KMIPProxy(config=config, config_file=opts.config_file)
     client.open()
 
     result = client.discover_versions(protocol_versions=protocol_versions)
