@@ -79,14 +79,13 @@ class CryptographyEngine(api.CryptographicEngine):
             enums.HashingAlgorithm.SHA_512: hashes.SHA512
         }
 
-        # GCM is supported by cryptography but requires inputs that are not
-        # supported by the KMIP spec. It is excluded for now.
         self._modes = {
             enums.BlockCipherMode.CBC: modes.CBC,
             enums.BlockCipherMode.ECB: modes.ECB,
             enums.BlockCipherMode.OFB: modes.OFB,
             enums.BlockCipherMode.CFB: modes.CFB,
-            enums.BlockCipherMode.CTR: modes.CTR
+            enums.BlockCipherMode.CTR: modes.CTR,
+            enums.BlockCipherMode.GCM: modes.GCM
         }
         self._asymmetric_padding_methods = {
              enums.PaddingMethod.OAEP:     asymmetric_padding.OAEP,
@@ -294,6 +293,8 @@ class CryptographyEngine(api.CryptographicEngine):
                 cipher_mode=None,
                 padding_method=None,
                 iv_nonce=None,
+                auth_additional_data=None,
+                auth_tag_length=None,
                 hashing_algorithm=None):
         """
         Encrypt data using symmetric or asymmetric encryption.
@@ -316,6 +317,12 @@ class CryptographyEngine(api.CryptographicEngine):
                 of the encryption algorithm. Optional, defaults to None. If
                 required and not provided, it will be autogenerated and
                 returned with the cipher text.
+            auth_additional_data (bytes): Any additional data to be
+                authenticated via the Authenticated Encryption Tag.
+                Optional, defaults to None.
+            auth_tag_length (int): The length of the authentication tag in
+                bytes. This parameter SHALL be provided when the Block
+                Cipher Mode is GCM.
             hashing_algorithm (HashingAlgorithm): An enumeration specifying
                 the hashing algorithm to use with the encryption algorithm,
                 if needed. Required for OAEP-based asymmetric encryption.
@@ -328,6 +335,8 @@ class CryptographyEngine(api.CryptographicEngine):
                 * iv_nonce - the bytes of the IV/counter/nonce used if it
                     was needed by the encryption scheme and if it was
                     automatically generated for the encryption
+                * auth_tag - the bytes of the authentication tag used in GCM or
+                    CCM mode
 
         Raises:
             InvalidField: Raised when the algorithm is unsupported or the
@@ -373,7 +382,9 @@ class CryptographyEngine(api.CryptographicEngine):
                 plain_text,
                 cipher_mode=cipher_mode,
                 padding_method=padding_method,
-                iv_nonce=iv_nonce
+                iv_nonce=iv_nonce,
+                auth_additional_data=auth_additional_data,
+                auth_tag_length=auth_tag_length
             )
 
     def _encrypt_symmetric(
@@ -383,7 +394,9 @@ class CryptographyEngine(api.CryptographicEngine):
             plain_text,
             cipher_mode=None,
             padding_method=None,
-            iv_nonce=None):
+            iv_nonce=None,
+            auth_additional_data=None,
+            auth_tag_length=None):
         """
         Encrypt data using symmetric encryption.
 
@@ -406,6 +419,12 @@ class CryptographyEngine(api.CryptographicEngine):
                 of the encryption algorithm. Optional, defaults to None. If
                 required and not provided, it will be autogenerated and
                 returned with the cipher text.
+            auth_additional_data (bytes): Any additional data to be
+                authenticated via the Authenticated Encryption Tag.
+                Optional, defaults to None.
+            auth_tag_length (int): The length of the authentication tag in
+                bytes. This parameter SHALL be provided when the Block Cipher
+                Mode is GCM.
 
         Returns:
             dict: A dictionary containing the encrypted data, with at least
@@ -414,6 +433,8 @@ class CryptographyEngine(api.CryptographicEngine):
                 * iv_nonce - the bytes of the IV/counter/nonce used if it
                     was needed by the encryption scheme and if it was
                     automatically generated for the encryption
+                * auth_tag - the bytes of the authentication tag used in
+                    GCM mode
 
         Raises:
             InvalidField: Raised when the algorithm is unsupported or the
@@ -440,6 +461,18 @@ class CryptographyEngine(api.CryptographicEngine):
                 "Invalid key bytes for the specified encryption algorithm."
             )
 
+        is_gcm_mode = cipher_mode == enums.BlockCipherMode.GCM
+        if not is_gcm_mode and auth_additional_data is not None:
+            raise exceptions.InvalidField(
+                'Authenticated encryption additional data is supported '
+                'in GCM mode only.'
+            )
+        if is_gcm_mode and auth_tag_length is None:
+            raise exceptions.InvalidField(
+                'Authenticated encryption tag length must be provided '
+                'in GCM mode only.'
+            )
+
         # Set up the cipher mode if needed
         return_iv_nonce = False
         if encryption_algorithm == enums.CryptographicAlgorithm.RC4:
@@ -459,7 +492,10 @@ class CryptographyEngine(api.CryptographicEngine):
                 if iv_nonce is None:
                     iv_nonce = os.urandom(algorithm.block_size // 8)
                     return_iv_nonce = True
-                mode = mode(iv_nonce)
+                if is_gcm_mode:
+                    mode = mode(iv_nonce, None, min_tag_length=auth_tag_length)
+                else:
+                    mode = mode(iv_nonce)
             else:
                 mode = mode()
 
@@ -477,15 +513,16 @@ class CryptographyEngine(api.CryptographicEngine):
         # Encrypt the plain text
         cipher = ciphers.Cipher(algorithm, mode, backend=default_backend())
         encryptor = cipher.encryptor()
+        if auth_additional_data is not None:
+            encryptor.authenticate_additional_data(auth_additional_data)
         cipher_text = encryptor.update(plain_text) + encryptor.finalize()
 
+        result = {'cipher_text': cipher_text}
         if return_iv_nonce:
-            return {
-                'cipher_text': cipher_text,
-                'iv_nonce': iv_nonce
-            }
-        else:
-            return {'cipher_text': cipher_text}
+            result['iv_nonce'] = iv_nonce
+        if is_gcm_mode:
+            result['auth_tag'] = encryptor.tag[:auth_tag_length]
+        return result
 
     def _encrypt_asymmetric(self,
                             encryption_algorithm,
@@ -612,6 +649,8 @@ class CryptographyEngine(api.CryptographicEngine):
                 cipher_mode=None,
                 padding_method=None,
                 iv_nonce=None,
+                auth_additional_data=None,
+                auth_tag=None,
                 hashing_algorithm=None):
         """
         Decrypt data using symmetric decryption.
@@ -633,6 +672,14 @@ class CryptographyEngine(api.CryptographicEngine):
                 Optional otherwise, defaults to None.
             iv_nonce (bytes): The IV/nonce value to use to initialize the mode
                 of the decryption algorithm. Optional, defaults to None.
+            auth_additional_data (bytes): Any additional data to be
+                authenticated via the Authenticated Encryption Tag.
+                Added in KMIP 1.4.
+            auth_tag (bytes): Specifies the tag that will be needed to
+                authenticate the decrypted data. Only returned on completion
+                of the encryption of the last of the plaintext by an
+                authenticated encryption cipher. Optional, defaults to None.
+                Added in KMIP 1.4.
             hashing_algorithm (HashingAlgorithm): An enumeration specifying
                 the hashing algorithm to use with the decryption algorithm,
                 if needed. Required for OAEP-based asymmetric decryption.
@@ -687,7 +734,9 @@ class CryptographyEngine(api.CryptographicEngine):
                 cipher_text,
                 cipher_mode=cipher_mode,
                 padding_method=padding_method,
-                iv_nonce=iv_nonce
+                iv_nonce=iv_nonce,
+                auth_additional_data=auth_additional_data,
+                auth_tag=auth_tag
             )
 
     def _decrypt_symmetric(
@@ -697,7 +746,9 @@ class CryptographyEngine(api.CryptographicEngine):
             cipher_text,
             cipher_mode=None,
             padding_method=None,
-            iv_nonce=None):
+            iv_nonce=None,
+            auth_additional_data=None,
+            auth_tag=None):
         """
         Decrypt data using symmetric decryption.
 
@@ -718,6 +769,14 @@ class CryptographyEngine(api.CryptographicEngine):
                 Optional otherwise, defaults to None.
             iv_nonce (bytes): The IV/nonce value to use to initialize the mode
                 of the decryption algorithm. Optional, defaults to None.
+            auth_additional_data (bytes): Any additional data to be
+                authenticated via the Authenticated Encryption Tag.
+                Added in KMIP 1.4.
+            auth_tag (bytes): Specifies the tag that will be needed to
+                authenticate the decrypted data. Only returned on completion
+                of the encryption of the last of the plaintext by an
+                authenticated encryption cipher. Optional, defaults to None.
+                Added in KMIP 1.4.
 
         Returns:
             bytes: the bytes of the decrypted data
@@ -746,6 +805,16 @@ class CryptographyEngine(api.CryptographicEngine):
                 "Invalid key bytes for the specified decryption algorithm."
             )
 
+        is_gcm_mode = cipher_mode == enums.BlockCipherMode.GCM
+        if auth_additional_data is not None and not is_gcm_mode:
+            raise exceptions.InvalidField(
+                'Additional data is supported in GCM mode only.'
+            )
+        if is_gcm_mode and auth_tag is None:
+            raise exceptions.InvalidField(
+                'Authenticated tag must be provided in GCM mode.'
+            )
+
         # Set up the cipher mode if needed
         if decryption_algorithm == enums.CryptographicAlgorithm.RC4:
             mode = None
@@ -765,13 +834,22 @@ class CryptographyEngine(api.CryptographicEngine):
                     raise exceptions.InvalidField(
                         "IV/nonce is required."
                     )
-                mode = mode(iv_nonce)
+                if is_gcm_mode:
+                    mode = mode(
+                        iv_nonce,
+                        tag=auth_tag,
+                        min_tag_length=len(auth_tag)
+                    )
+                else:
+                    mode = mode(iv_nonce)
             else:
                 mode = mode()
 
         # Decrypt the plain text
         cipher = ciphers.Cipher(algorithm, mode, backend=default_backend())
         decryptor = cipher.decryptor()
+        if auth_additional_data is not None:
+            decryptor.authenticate_additional_data(auth_additional_data)
         plain_text = decryptor.update(cipher_text) + decryptor.finalize()
 
         # Unpad the plain text if needed (separate methods for testing
@@ -797,7 +875,7 @@ class CryptographyEngine(api.CryptographicEngine):
             padding_method,
             hashing_algorithm=None):
         """
-        Encrypt data using asymmetric decryption.
+        Decrypt data using asymmetric decryption.
 
         Args:
             decryption_algorithm (CryptographicAlgorithm): An enumeration
