@@ -13,20 +13,25 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import logging
 import logging.handlers as handlers
+import multiprocessing
 import optparse
 import os
 import signal
+import six
 import socket
 import ssl
 import sys
 import threading
 
 from kmip.core import exceptions
+from kmip.core import policy as operation_policy
 from kmip.services import auth
 from kmip.services.server import config
 from kmip.services.server import engine
+from kmip.services.server import monitor
 from kmip.services.server import session
 
 
@@ -40,6 +45,7 @@ class KmipServer(object):
     shutting down all server components upon receiving a termination signal.
     """
 
+    # TODO (peter-hamilton) Move to using **kwargs for all server parameters.
     def __init__(
             self,
             hostname=None,
@@ -53,7 +59,9 @@ class KmipServer(object):
             policy_path=None,
             enable_tls_client_auth=None,
             tls_cipher_suites=None,
-            logging_level=None
+            logging_level=None,
+            live_policies=False,
+            database_path=None
     ):
         """
         Create a KmipServer.
@@ -113,6 +121,12 @@ class KmipServer(object):
                 level for the server. All log messages logged at this level or
                 higher in criticality will be logged. All log messages lower
                 in criticality will not be logged. Optional, defaults to None.
+            live_policies (boolean): A boolean indicating if the operation
+                policy directory should be actively monitored to autoload any
+                policy changes while the server is running. Optional, defaults
+                to False.
+            database_path (string): The path to the server's SQLite database
+                file. Optional, defaults to None.
         """
         self._logger = logging.getLogger('kmip.server')
         self._setup_logging(log_path)
@@ -129,8 +143,11 @@ class KmipServer(object):
             policy_path,
             enable_tls_client_auth,
             tls_cipher_suites,
-            logging_level
+            logging_level,
+            database_path
         )
+        self.live_policies = live_policies
+        self.policies = {}
 
         self._logger.setLevel(self.config.settings.get('logging_level'))
 
@@ -140,9 +157,6 @@ class KmipServer(object):
         else:
             self.auth_suite = auth.BasicAuthenticationSuite(cipher_suites)
 
-        self._engine = engine.KmipEngine(
-            self.config.settings.get('policy_path')
-        )
         self._session_id = 1
         self._is_serving = False
 
@@ -179,7 +193,8 @@ class KmipServer(object):
             policy_path=None,
             enable_tls_client_auth=None,
             tls_cipher_suites=None,
-            logging_level=None
+            logging_level=None,
+            database_path=None
     ):
         if path:
             self.config.load_settings(path)
@@ -210,6 +225,8 @@ class KmipServer(object):
             )
         if logging_level:
             self.config.set_setting('logging_level', logging_level)
+        if database_path:
+            self.config.set_setting('database_path', database_path)
 
     def start(self):
         """
@@ -223,9 +240,34 @@ class KmipServer(object):
             NetworkingError: Raised if the TLS socket cannot be bound to the
                 network address.
         """
+        self.manager = multiprocessing.Manager()
+        self.policies = self.manager.dict()
+        policies = copy.deepcopy(operation_policy.policies)
+        for policy_name, policy_set in six.iteritems(policies):
+            self.policies[policy_name] = policy_set
+
+        self.policy_monitor = monitor.PolicyDirectoryMonitor(
+            self.config.settings.get('policy_path'),
+            self.policies,
+            self.live_policies
+        )
+
+        def interrupt_handler(trigger, frame):
+            self.policy_monitor.stop()
+        signal.signal(signal.SIGINT, interrupt_handler)
+        signal.signal(signal.SIGTERM, interrupt_handler)
+
+        self.policy_monitor.start()
+
+        self._engine = engine.KmipEngine(
+            policies=self.policies,
+            database_path=self.config.settings.get('database_path')
+        )
+
         self._logger.info("Starting server socket handler.")
 
         # Create a TCP stream socket and configure it for immediate reuse.
+        socket.setdefaulttimeout(10)
         self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 
@@ -253,7 +295,7 @@ class KmipServer(object):
             cert_reqs=ssl.CERT_REQUIRED,
             ssl_version=self.auth_suite.protocol,
             ca_certs=self.config.settings.get('ca_path'),
-            do_handshake_on_connect=True,
+            do_handshake_on_connect=False,
             suppress_ragged_eofs=True,
             ciphers=self.auth_suite.ciphers
         )
@@ -328,6 +370,16 @@ class KmipServer(object):
                 "Server failed to shutdown socket handler."
             )
 
+        if hasattr(self, "policy_monitor"):
+            try:
+                self.policy_monitor.stop()
+                self.policy_monitor.join()
+            except Exception as e:
+                self._logger.exception(e)
+                raise exceptions.ShutdownError(
+                    "Server failed to clean up the policy monitor."
+                )
+
     def serve(self):
         """
         Serve client connections.
@@ -356,6 +408,11 @@ class KmipServer(object):
         while self._is_serving:
             try:
                 connection, address = self._socket.accept()
+            except socket.timeout:
+                # Setting the default socket timeout to break hung connections
+                # will cause accept to periodically raise socket.timeout. This
+                # is expected behavior, so ignore it and retry accept.
+                pass
             except socket.error as e:
                 self._logger.warning(
                     "Error detected while establishing new connection."
@@ -398,10 +455,12 @@ class KmipServer(object):
             s = session.KmipSession(
                 self._engine,
                 connection,
+                address,
                 name=session_name,
                 enable_tls_client_auth=self.config.settings.get(
                     'enable_tls_client_auth'
-                )
+                ),
+                auth_settings=self.config.settings.get('auth_plugins')
             )
             s.daemon = True
             s.start()
@@ -560,6 +619,18 @@ def build_argument_parser():
             "DEBUG, INFO). Optional, defaults to None."
         )
     )
+    parser.add_option(
+        "-d",
+        "--database_path",
+        action="store",
+        type="str",
+        default=None,
+        dest="database_path",
+        help=(
+            "A string representing a path to the server's SQLite database "
+            "file. Optional, defaults to None."
+        ),
+    )
 
     return parser
 
@@ -592,6 +663,10 @@ def main(args=None):
         kwargs['enable_tls_client_auth'] = False
     if opts.logging_level:
         kwargs['logging_level'] = opts.logging_level
+    if opts.database_path:
+        kwargs['database_path'] = opts.database_path
+
+    kwargs['live_policies'] = True
 
     # Create and start the server.
     s = KmipServer(**kwargs)
