@@ -35,8 +35,8 @@ from kmip.core.enums import KeyFormatType as KeyFormatTypeEnum
 from kmip.core.enums import ResultStatus
 from kmip.core.enums import ResultReason
 
-from kmip.core.errors import KMIPServerSuicideError
-from kmip.core.errors import KMIPServerZombieError
+from kmip.core.exceptions import KMIPServerSuicideError
+from kmip.core.exceptions import KMIPServerZombieError
 
 from kmip.core.factories.attributes import AttributeFactory
 from kmip.core.factories.credentials import CredentialFactory
@@ -104,9 +104,9 @@ class TestKMIPClientIntegration(TestCase):
 
         self._check_result_status(result.result_status.value, ResultStatus,
                                   ResultStatus.SUCCESS)
-        self._check_object_type(result.object_type.value, ObjectType,
+        self._check_object_type(result.object_type, ObjectType,
                                 ObjectType.SYMMETRIC_KEY)
-        self._check_uuid(result.uuid.value, str)
+        self._check_uuid(result.uuid, str)
 
         # Check the template attribute type
         self._check_template_attribute(result.template_attribute,
@@ -122,7 +122,7 @@ class TestKMIPClientIntegration(TestCase):
         credential = self.cred_factory.create_credential(credential_type,
                                                          credential_value)
         result = self._create_symmetric_key()
-        uuid = result.uuid.value
+        uuid = result.uuid
 
         result = self.client.get(uuid=uuid, credential=credential)
 
@@ -146,7 +146,7 @@ class TestKMIPClientIntegration(TestCase):
         credential = self.cred_factory.create_credential(credential_type,
                                                          credential_value)
         result = self._create_symmetric_key()
-        uuid = result.uuid.value
+        uuid = result.uuid
 
         # Verify the secret was created
         result = self.client.get(uuid=uuid, credential=credential)
@@ -230,7 +230,7 @@ class TestKMIPClientIntegration(TestCase):
 
         self._check_result_status(result.result_status.value, ResultStatus,
                                   ResultStatus.SUCCESS)
-        self._check_uuid(result.uuid.value, str)
+        self._check_uuid(result.uuid, str)
 
         # Check the template attribute type
         self._check_template_attribute(result.template_attribute,
@@ -238,7 +238,7 @@ class TestKMIPClientIntegration(TestCase):
                                        [[str, 'Unique Identifier', str,
                                          None]])
         # Check that the returned key bytes match what was provided
-        uuid = result.uuid.value
+        uuid = result.uuid
         result = self.client.get(uuid=uuid, credential=credential)
 
         self._check_result_status(result.result_status.value, ResultStatus,
