@@ -30,10 +30,10 @@ class TestDiscoverVersionsRequestPayload(TestCase):
 
         self.protocol_versions_empty = list()
         self.protocol_versions_one = list()
-        self.protocol_versions_one.append(ProtocolVersion.create(1, 0))
+        self.protocol_versions_one.append(ProtocolVersion(1, 0))
         self.protocol_versions_two = list()
-        self.protocol_versions_two.append(ProtocolVersion.create(1, 1))
-        self.protocol_versions_two.append(ProtocolVersion.create(1, 0))
+        self.protocol_versions_two.append(ProtocolVersion(1, 1))
+        self.protocol_versions_two.append(ProtocolVersion(1, 0))
 
         self.encoding_empty = utils.BytearrayStream((
             b'\x42\x00\x79\x01\x00\x00\x00\x00'))
@@ -62,13 +62,13 @@ class TestDiscoverVersionsRequestPayload(TestCase):
 
     def test_validate_with_invalid_protocol_versions(self):
         kwargs = {'protocol_versions': 'invalid'}
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             TypeError, "invalid protocol versions list",
             payloads.DiscoverVersionsRequestPayload, **kwargs)
 
     def test_validate_with_invalid_protocol_version(self):
         kwargs = {'protocol_versions': ['invalid']}
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             TypeError, "invalid protocol version",
             payloads.DiscoverVersionsRequestPayload, **kwargs)
 
@@ -157,10 +157,10 @@ class TestDiscoverVersionsResponsePayload(TestCase):
 
         self.protocol_versions_empty = list()
         self.protocol_versions_one = list()
-        self.protocol_versions_one.append(ProtocolVersion.create(1, 0))
+        self.protocol_versions_one.append(ProtocolVersion(1, 0))
         self.protocol_versions_two = list()
-        self.protocol_versions_two.append(ProtocolVersion.create(1, 1))
-        self.protocol_versions_two.append(ProtocolVersion.create(1, 0))
+        self.protocol_versions_two.append(ProtocolVersion(1, 1))
+        self.protocol_versions_two.append(ProtocolVersion(1, 0))
 
         self.encoding_empty = utils.BytearrayStream((
             b'\x42\x00\x7C\x01\x00\x00\x00\x00'))
@@ -189,13 +189,13 @@ class TestDiscoverVersionsResponsePayload(TestCase):
 
     def test_validate_with_invalid_protocol_versions(self):
         kwargs = {'protocol_versions': 'invalid'}
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             TypeError, "invalid protocol versions list",
             payloads.DiscoverVersionsResponsePayload, **kwargs)
 
     def test_validate_with_invalid_protocol_version(self):
         kwargs = {'protocol_versions': ['invalid']}
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             TypeError, "invalid protocol version",
             payloads.DiscoverVersionsResponsePayload, **kwargs)
 
