@@ -989,15 +989,18 @@ class TestGetAttributesResponsePayload(testtools.TestCase):
                 attribute_name=objects.Attribute.AttributeName(
                     'Object Group'
                 ),
-                attribute_value=attributes.ObjectGroup('Group1')
+                attribute_value=primitives.TextString(
+                    "Group1",
+                    enums.Tags.OBJECT_GROUP
+                )
             ),
             objects.Attribute(
                 attribute_name=objects.Attribute.AttributeName(
                     'Application Specific Information'
                 ),
                 attribute_value=attributes.ApplicationSpecificInformation(
-                    attributes.ApplicationNamespace('ssl'),
-                    attributes.ApplicationData('www.example.com')
+                    application_namespace="ssl",
+                    application_data="www.example.com"
                 )
             ),
             objects.Attribute(
@@ -1166,8 +1169,8 @@ class TestGetAttributesResponsePayload(testtools.TestCase):
                     'Application Specific Information'
                 ),
                 attribute_value=attributes.ApplicationSpecificInformation(
-                    attributes.ApplicationNamespace('ssl'),
-                    attributes.ApplicationData('www.example.com')
+                    application_namespace="ssl",
+                    application_data="www.example.com"
                 )
             ),
             payload.attributes
@@ -1271,8 +1274,8 @@ class TestGetAttributesResponsePayload(testtools.TestCase):
                         'Application Specific Information'
                     ),
                     attribute_value=attributes.ApplicationSpecificInformation(
-                        attributes.ApplicationNamespace('ssl'),
-                        attributes.ApplicationData('www.example.com')
+                        application_namespace="ssl",
+                        application_data="www.example.com"
                     )
                 ),
                 objects.Attribute(
